@@ -662,106 +662,13 @@ out:
 	return error;
 }
 
-/* These are basically cman overrides to the totem config bits */
-static void add_cman_overrides(struct objdb_iface_ver0 *objdb)
+static void add_logging_overrides(struct objdb_iface_ver0 *objdb)
 {
 	char *logstr;
 	char *logfacility;
 	char *loglevel;
 	hdb_handle_t object_handle;
 	hdb_handle_t find_handle;
-	char tmp[256];
-
-	/* "totem" key already exists, because we have added the interfaces by now */
-	objdb->object_find_create(OBJECT_PARENT_HANDLE,"totem", strlen("totem"), &find_handle);
-	if (objdb->object_find_next(find_handle, &object_handle) == 0)
-	{
-		char *value;
-
-		objdb->object_key_create_typed(object_handle, "version",
-					       "2", 2, OBJDB_VALUETYPE_STRING);
-
-		sprintf(tmp, "%d", nodeid);
-		objdb->object_key_create_typed(object_handle, "nodeid",
-					       tmp, strlen(tmp)+1, OBJDB_VALUETYPE_STRING);
-
-		objdb->object_key_create_typed(object_handle, "vsftype",
-					       "none", strlen("none")+1, OBJDB_VALUETYPE_STRING);
-
-		/* Set the token timeout is 10 seconds, but don't overrride anything that
-		   might be in cluster.conf */
-		if (objdb_get_string(objdb, object_handle, "token", &value)) {
-			snprintf(tmp, sizeof(tmp), "%d", DEFAULT_TOKEN_TIMEOUT);
-			objdb->object_key_create_typed(object_handle, "token",
-						       tmp, strlen(tmp)+1, OBJDB_VALUETYPE_STRING);
-		}
-
-		/* Extend consensus & join timeouts per bz#214290 */
-		if (objdb_get_string(objdb, object_handle, "join", &value)) {
-			objdb->object_key_create_typed(object_handle, "join",
-						       "60", strlen("60")+1, OBJDB_VALUETYPE_STRING);
-		}
-		/* Extend fail_to_recv_const, see bz#587078 */
-		if (objdb_get_string(objdb, object_handle, "fail_recv_const", &value)) {
-			objdb->object_key_create_typed(object_handle, "fail_recv_const",
-						       "2500", strlen("2500")+1, OBJDB_VALUETYPE_STRING);
-		}
-		/* consensus should be 2*token, see bz#544482*/
-		if (objdb_get_string(objdb, object_handle, "consensus", &value)) {
-		        unsigned int token;
-			char calc_consensus[32];
-
-			objdb_get_int(objdb, object_handle, "token", &token, DEFAULT_TOKEN_TIMEOUT);
-			sprintf(calc_consensus, "%d", token*2);
-			objdb->object_key_create_typed(object_handle, "consensus",
-						       calc_consensus, strlen(calc_consensus)+1, OBJDB_VALUETYPE_STRING);
-		}
-
-		/* Set RRP mode appropriately */
-		if (objdb_get_string(objdb, object_handle, "rrp_mode", &value)) {
-			if (num_interfaces > 1) {
-				objdb->object_key_create_typed(object_handle, "rrp_mode",
-							       "active", strlen("active")+1, OBJDB_VALUETYPE_STRING);
-			}
-			else {
-				objdb->object_key_create_typed(object_handle, "rrp_mode",
-							       "none", strlen("none")+1, OBJDB_VALUETYPE_STRING);
-			}
-		}
-
-		if (objdb_get_string(objdb, object_handle, "secauth", &value)) {
-			sprintf(tmp, "%d", 1);
-			objdb->object_key_create_typed(object_handle, "secauth",
-						       tmp, strlen(tmp)+1, OBJDB_VALUETYPE_STRING);
-		}
-
-		/* optional security key filename */
-		if (!key_filename) {
-			objdb_get_string(objdb, object_handle, "keyfile", &key_filename);
-		}
-		else {
-			objdb->object_key_create_typed(object_handle, "keyfile",
-						       key_filename, strlen(key_filename)+1, OBJDB_VALUETYPE_STRING);
-		}
-		if (!key_filename) {
-			/* Use the cluster name as key,
-			 * This isn't a good isolation strategy but it does make sure that
-			 * clusters on the same port/multicast by mistake don't actually interfere
-			 * and that we have some form of encryption going.
-			 */
-
-			int keylen;
-			memset(tmp, 0, sizeof(tmp));
-
-			strcpy(tmp, cluster_name);
-
-			/* Key length must be a multiple of 4 */
-			keylen = (strlen(cluster_name)+4) & 0xFC;
-			objdb->object_key_create_typed(object_handle, "key",
-						       tmp, keylen, OBJDB_VALUETYPE_STRING);
-		}
-	}
-	objdb->object_find_destroy(find_handle);
 
 	/* Make sure mainconfig doesn't stomp on our logging options */
 	objdb->object_find_create(OBJECT_PARENT_HANDLE, "logging", strlen("logging"), &find_handle);
@@ -840,6 +747,154 @@ static void add_cman_overrides(struct objdb_iface_ver0 *objdb)
 					       "yes", strlen("yes")+1, OBJDB_VALUETYPE_STRING);
 	}
 
+
+}
+
+static int count_configured_nodes(struct objdb_iface_ver0 *objdb)
+{
+	hdb_handle_t find_handle = 0;
+	hdb_handle_t nodes_handle;
+	int count = 0;
+
+	nodes_handle = nodeslist_init(objdb, cluster_parent_handle, &find_handle);
+	while (nodes_handle) {
+		count++;
+		nodes_handle = nodeslist_next(objdb, find_handle);
+	}
+	objdb->object_find_destroy(find_handle);
+
+	return count;
+}
+
+/* These are basically cman overrides to the totem config bits */
+static void add_cman_overrides(struct objdb_iface_ver0 *objdb)
+{
+	hdb_handle_t object_handle;
+	hdb_handle_t find_handle;
+	int node_count = 0;
+	char tmp[256];
+
+	/* "totem" key already exists, because we have added the interfaces by now */
+	objdb->object_find_create(OBJECT_PARENT_HANDLE,"totem", strlen("totem"), &find_handle);
+	if (objdb->object_find_next(find_handle, &object_handle) == 0)
+	{
+		char *value;
+
+		objdb->object_key_create_typed(object_handle, "version",
+					       "2", 2, OBJDB_VALUETYPE_STRING);
+
+		sprintf(tmp, "%d", nodeid);
+		objdb->object_key_create_typed(object_handle, "nodeid",
+					       tmp, strlen(tmp)+1, OBJDB_VALUETYPE_STRING);
+
+		objdb->object_key_create_typed(object_handle, "vsftype",
+					       "none", strlen("none")+1, OBJDB_VALUETYPE_STRING);
+
+		/* Set the token timeout is 10 seconds, but don't overrride anything that
+		   might be in cluster.conf */
+		if (objdb_get_string(objdb, object_handle, "token", &value)) {
+			snprintf(tmp, sizeof(tmp), "%d", DEFAULT_TOKEN_TIMEOUT);
+			objdb->object_key_create_typed(object_handle, "token",
+						       tmp, strlen(tmp)+1, OBJDB_VALUETYPE_STRING);
+		}
+
+		/* Extend join timeouts per bz#214290 */
+		if (objdb_get_string(objdb, object_handle, "join", &value)) {
+			objdb->object_key_create_typed(object_handle, "join",
+						       "60", strlen("60")+1, OBJDB_VALUETYPE_STRING);
+		}
+		/* Extend fail_to_recv_const, see bz#587078 */
+		if (objdb_get_string(objdb, object_handle, "fail_recv_const", &value)) {
+			objdb->object_key_create_typed(object_handle, "fail_recv_const",
+						       "2500", strlen("2500")+1, OBJDB_VALUETYPE_STRING);
+		}
+
+		/*
+		 * consensus should be:
+		 * 2 nodes   - 200 ms <= consensus = token * 0.2 <= 2000
+		 * > 2 nodes - consensus = token + 2000
+		 *
+		 * autoconfig clusters will work as > 2 nodes
+		 *
+		 * See 611391#c19
+		 */
+
+		node_count=count_configured_nodes(objdb);
+
+		/* if we are running in autoconfig or we can't count the nodes, then play safe */
+		if ((getenv("CMAN_NOCONFIG")) || (node_count == 0))
+			node_count=3;
+
+		if (objdb_get_string(objdb, object_handle, "consensus", &value)) {
+			unsigned int token=0;
+			unsigned int consensus;
+			char calc_consensus[32];
+
+			objdb_get_int(objdb, object_handle, "token", &token, DEFAULT_TOKEN_TIMEOUT);
+
+			if (node_count > 2) {
+				consensus = (float)token+2000;
+			} else {
+				consensus = (float)token*0.2;
+				if (consensus < 200)
+					consensus = 200;
+				if (consensus > 2000)
+					consensus = 2000;
+			}
+
+			snprintf(calc_consensus, sizeof(calc_consensus), "%d", consensus);
+			objdb->object_key_create_typed(object_handle, "consensus",
+						       calc_consensus, strlen(calc_consensus)+1, OBJDB_VALUETYPE_STRING);
+		}
+
+		/* Set RRP mode appropriately */
+		if (objdb_get_string(objdb, object_handle, "rrp_mode", &value)) {
+			if (num_interfaces > 1) {
+				objdb->object_key_create_typed(object_handle, "rrp_mode",
+							       "active", strlen("active")+1, OBJDB_VALUETYPE_STRING);
+			}
+			else {
+				objdb->object_key_create_typed(object_handle, "rrp_mode",
+							       "none", strlen("none")+1, OBJDB_VALUETYPE_STRING);
+			}
+		}
+
+		if (objdb_get_string(objdb, object_handle, "secauth", &value)) {
+			sprintf(tmp, "%d", 1);
+			objdb->object_key_create_typed(object_handle, "secauth",
+						       tmp, strlen(tmp)+1, OBJDB_VALUETYPE_STRING);
+		}
+
+		/* optional security key filename */
+		if (!key_filename) {
+			objdb_get_string(objdb, object_handle, "keyfile", &key_filename);
+		}
+		else {
+			objdb->object_key_create_typed(object_handle, "keyfile",
+						       key_filename, strlen(key_filename)+1, OBJDB_VALUETYPE_STRING);
+		}
+		if (!key_filename) {
+			/* Use the cluster name as key,
+			 * This isn't a good isolation strategy but it does make sure that
+			 * clusters on the same port/multicast by mistake don't actually interfere
+			 * and that we have some form of encryption going.
+			 */
+
+			int keylen;
+			memset(tmp, 0, sizeof(tmp));
+
+			strcpy(tmp, cluster_name);
+
+			/* Key length must be a multiple of 4 */
+			keylen = (strlen(cluster_name)+4) & 0xFC;
+			objdb->object_key_create_typed(object_handle, "key",
+						       tmp, keylen, OBJDB_VALUETYPE_STRING);
+		}
+	}
+	objdb->object_find_destroy(find_handle);
+
+	add_logging_overrides(objdb);
+
 	/* Make sure we allow connections from user/group "ais" */
 	objdb->object_find_create(OBJECT_PARENT_HANDLE, "aisexec", strlen("aisexec"), &find_handle);
 	if (objdb->object_find_next(find_handle, &object_handle) != 0) {
@@ -1066,7 +1121,7 @@ static int copy_config_tree(struct objdb_iface_ver0 *objdb, hdb_handle_t source_
 	while ( (res = objdb->object_find_next(find_handle, &object_handle) == 0)) {
 
 		/* Down we go ... */
-		copy_config_tree(objdb, object_handle, new_object, 0);
+		copy_config_tree(objdb, object_handle, new_object, always_create);
 	}
 	objdb->object_find_destroy(find_handle);
 
@@ -1137,6 +1192,8 @@ static int cmanpre_reloadconfig(struct objdb_iface_ver0 *objdb, int flush, const
 	hdb_handle_t object_handle;
 	hdb_handle_t find_handle;
 	hdb_handle_t cluster_parent_handle_new;
+	unsigned int config_version = 0, config_version_new = 0;
+	char *config_value = NULL;
 
 	/* don't reload if we've been told to run configless */
 	if (getenv("CMAN_NOCONFIG")) {
@@ -1159,6 +1216,34 @@ static int cmanpre_reloadconfig(struct objdb_iface_ver0 *objdb, int flush, const
 	}
 	objdb->object_find_destroy(find_handle);
 
+	if (!objdb->object_key_get(cluster_parent_handle, "config_version", strlen("config_version"), (void *)&config_value, NULL)) {
+		if (config_value) {
+			config_version = atoi(config_value);
+		} else {
+			/* it should never ever happen.. */
+			sprintf (error_reason, "%s", "Cannot find old /cluster/config_version key in configuration\n");
+			goto err;
+		}
+	}
+
+	config_value = NULL;
+
+	if (!objdb->object_key_get(cluster_parent_handle_new, "config_version", strlen("config_version"), (void *)&config_value, NULL)) {
+		if (config_value) {
+			config_version_new = atoi(config_value);
+		} else {
+			objdb->object_destroy(cluster_parent_handle_new);
+			sprintf (error_reason, "%s", "Cannot find new /cluster/config_version key in configuration\n");
+			goto err;
+		}
+	}
+
+	if (config_version_new <= config_version) {
+		objdb->object_destroy(cluster_parent_handle_new);
+		sprintf (error_reason, "%s", "New configuration version has to be newer than current running configuration\n");
+		goto err;
+	}
+
 	/* destroy the old one */
 	objdb->object_destroy(cluster_parent_handle);
 
@@ -1169,12 +1254,12 @@ static int cmanpre_reloadconfig(struct objdb_iface_ver0 *objdb, int flush, const
 	objdb->object_find_create(OBJECT_PARENT_HANDLE, "logging", strlen("logging"), &find_handle);
 	ret = objdb->object_find_next(find_handle, &object_handle);
 	objdb->object_find_destroy(find_handle);
-	if (ret) {
+	if (!ret) {
 		objdb->object_destroy(object_handle);
 	}
 
 	/* copy /cluster/logging to /logging */
-	ret = copy_tree_to_root(objdb, "logging", 0);
+	ret = copy_tree_to_root(objdb, "logging", 1);
 
 	/* Note: we do NOT delete /totem as corosync stores other things in there that
 	   it needs! */
@@ -1182,6 +1267,8 @@ static int cmanpre_reloadconfig(struct objdb_iface_ver0 *objdb, int flush, const
 	/* copy /cluster/totem to /totem */
 	ret = copy_tree_to_root(objdb, "totem", 0);
 
+	add_logging_overrides(objdb);
+
 	return 0;
 
 err:
@@ -1296,7 +1383,7 @@ static int cmanpre_readconfig(struct objdb_iface_ver0 *objdb, const char **error
 	else {
 		/* Move these to a place where corosync expects to find them */
 		ret = copy_tree_to_root(objdb, "totem", 0);
-		ret = copy_tree_to_root(objdb, "logging", 0);
+		ret = copy_tree_to_root(objdb, "logging", 1);
 		ret = copy_tree_to_root(objdb, "event", 0);
 		ret = copy_tree_to_root(objdb, "amf", 0);
 		ret = copy_tree_to_root(objdb, "aisexec", 0);
