@@ -45,6 +45,7 @@ struct node {
 	int check_fs;
 	int fs_notified;
 	uint64_t add_time;
+	uint64_t fail_time;
 	uint64_t fence_time;	/* for debug */
 	uint64_t cluster_add_time;
 	uint64_t cluster_remove_time;
@@ -263,6 +264,7 @@ void dlm_send_message(struct lockspace *ls, char *buf, int len)
 	hd->global_id   = cpu_to_le32(ls->global_id);
 	hd->flags       = cpu_to_le32(hd->flags);
 	hd->msgdata     = cpu_to_le32(hd->msgdata);
+	hd->msgdata2    = cpu_to_le32(hd->msgdata2);
 
 	_send_message(ls->cpg_handle, buf, len, type);
 }
@@ -355,12 +357,13 @@ static void free_ls(struct lockspace *ls)
    has failed yet).
 
    So, check that:
-   1. has fenced fenced the node after it joined this lockspace?
+   1. has fenced fenced the node since we saw it fail?
    2. fenced has no outstanding fencing ops
 
    For 1:
-   - record the time of the first good start message we see from node X
    - node X fails
+   - we see node X fail and X has non-zero add_time,
+     set check_fencing and record the fail time
    - wait for X to be removed from all dlm cpg's  (probably not necessary)
    - check that the fencing time is later than the recorded time above
 
@@ -498,6 +501,7 @@ static void node_history_fail(struct lockspace *ls, int nodeid,
 		node->check_fencing = 1;
 		node->fence_time = 0;
 		node->fence_queries = 0;
+		node->fail_time = time(NULL);
 	}
 
 	/* fenced will take care of making sure the quorum value
@@ -506,8 +510,10 @@ static void node_history_fail(struct lockspace *ls, int nodeid,
 	if (cfgd_enable_quorum && !cfgd_enable_fencing)
 		node->check_quorum = 1;
 
-	if (ls->fs_registered)
+	if (ls->fs_registered) {
+		log_group(ls, "check_fs nodeid %d set", nodeid);
 		node->check_fs = 1;
+	}
 
 	node->removed_seq = cg->seq;	/* for queries */
 	node->failed_reason = reason;	/* for queries */
@@ -536,10 +542,16 @@ static int check_fencing_done(struct lockspace *ls)
 		if (rv < 0)
 			log_error("fenced_node_info error %d", rv);
 
-		if (last_fenced_time > node->add_time) {
-			log_group(ls, "check_fencing %d %llu fenced at %llu",
+		/* need >= not just > because in at least one case
+		   we've seen fenced_time within the same second as
+		   fail_time: with external fencing, e.g. fence_node */
+
+		if (last_fenced_time >= node->fail_time) {
+			log_group(ls, "check_fencing %d done "
+				  "add %llu fail %llu last %llu",
 				  node->nodeid,
 				  (unsigned long long)node->add_time,
+				  (unsigned long long)node->fail_time,
 				  (unsigned long long)last_fenced_time);
 			node->check_fencing = 0;
 			node->add_time = 0;
@@ -547,9 +559,11 @@ static int check_fencing_done(struct lockspace *ls)
 		} else {
 			if (!node->fence_queries ||
 			    node->fence_time != last_fenced_time) {
-				log_group(ls, "check_fencing %d not fenced "
-					  "add %llu fence %llu", node->nodeid,
+				log_group(ls, "check_fencing %d wait "
+					  "add %llu fail %llu last %llu",
+					  node->nodeid,
 					 (unsigned long long)node->add_time,
+					 (unsigned long long)node->fail_time,
 					 (unsigned long long)last_fenced_time);
 				node->fence_queries++;
 				node->fence_time = last_fenced_time;
@@ -635,6 +649,7 @@ static int check_fs_done(struct lockspace *ls)
 			continue;
 
 		if (node->fs_notified) {
+			log_group(ls, "check_fs nodeid %d clear", node->nodeid);
 			node->check_fs = 0;
 		} else {
 			log_group(ls, "check_fs nodeid %d needs fs notify",
@@ -1078,9 +1093,11 @@ static void receive_plocks_stored(struct lockspace *ls, struct dlm_header *hd,
 {
 	struct ls_info *li;
 	struct id_info *ids;
+	uint32_t sig;
 
-	log_group(ls, "receive_plocks_stored %d:%u need_plocks %d",
-		  hd->nodeid, hd->msgdata, ls->need_plocks);
+	log_group(ls, "receive_plocks_stored %d:%u flags %x sig %x "
+		  "need_plocks %d", hd->nodeid, hd->msgdata, hd->flags,
+		  hd->msgdata2, ls->need_plocks);
 
 	if (!ls->need_plocks)
 		return;
@@ -1106,14 +1123,25 @@ static void receive_plocks_stored(struct lockspace *ls, struct dlm_header *hd,
 		return;
 	}
 
-	retrieve_plocks(ls);
+	retrieve_plocks(ls, &sig);
+
+	if ((hd->flags & DLM_MFLG_PLOCK_SIG) && (sig != hd->msgdata2)) {
+		log_error("lockspace %s plock disabled our sig %x "
+			  "nodeid %d sig %x", ls->name, sig, hd->nodeid,
+			  hd->msgdata2);
+		ls->disable_plock = 1;
+		ls->need_plocks = 1; /* don't set HAVEPLOCK */
+		ls->save_plocks = 0;
+		return;
+	}
+
 	process_saved_plocks(ls);
 	ls->need_plocks = 0;
 	ls->save_plocks = 0;
 }
 
 static void send_info(struct lockspace *ls, struct change *cg, int type,
-		      uint32_t flags)
+		      uint32_t flags, uint32_t msgdata2)
 {
 	struct dlm_header *hd;
 	struct ls_info *li;
@@ -1143,6 +1171,7 @@ static void send_info(struct lockspace *ls, struct change *cg, int type,
 	hd->type = type;
 	hd->msgdata = cg->seq;
 	hd->flags = flags;
+	hd->msgdata2 = msgdata2;
 
 	if (ls->joining)
 		hd->flags |= DLM_MFLG_JOINING;
@@ -1167,10 +1196,11 @@ static void send_info(struct lockspace *ls, struct change *cg, int type,
 		id++;
 	}
 
-	log_group(ls, "send_%s cg %u flags %x counts %u %d %d %d %d",
+	log_group(ls, "send_%s cg %u flags %x data2 %x counts %u %d %d %d %d",
 		  type == DLM_MSG_START ? "start" : "plocks_stored",
-		  cg->seq, hd->flags, ls->started_count, cg->member_count,
-		  cg->joined_count, cg->remove_count, cg->failed_count);
+		  cg->seq, hd->flags, hd->msgdata2, ls->started_count,
+		  cg->member_count, cg->joined_count, cg->remove_count,
+		  cg->failed_count);
 
 	dlm_send_message(ls, buf, len);
 
@@ -1181,14 +1211,14 @@ static void send_start(struct lockspace *ls)
 {
 	struct change *cg = list_first_entry(&ls->changes, struct change, list);
 
-	send_info(ls, cg, DLM_MSG_START, 0);
+	send_info(ls, cg, DLM_MSG_START, 0, 0);
 }
 
-static void send_plocks_stored(struct lockspace *ls)
+static void send_plocks_stored(struct lockspace *ls, uint32_t sig)
 {
 	struct change *cg = list_first_entry(&ls->changes, struct change, list);
 
-	send_info(ls, cg, DLM_MSG_PLOCKS_STORED, 0);
+	send_info(ls, cg, DLM_MSG_PLOCKS_STORED, DLM_MFLG_PLOCK_SIG, sig);
 }
 
 static int same_members(struct change *cg1, struct change *cg2)
@@ -1215,7 +1245,7 @@ static void send_nacks(struct lockspace *ls, struct change *startcg)
 		    same_members(cg, startcg)) {
 			log_group(ls, "send nack old cg %u new cg %u",
 				   cg->seq, startcg->seq);
-			send_info(ls, cg, DLM_MSG_START, DLM_MFLG_NACK);
+			send_info(ls, cg, DLM_MSG_START, DLM_MFLG_NACK, 0);
 		}
 	}
 }
@@ -1235,8 +1265,9 @@ static void prepare_plocks(struct lockspace *ls)
 {
 	struct change *cg = list_first_entry(&ls->changes, struct change, list);
 	struct member *memb;
+	uint32_t sig;
 
-	if (!cfgd_enable_plock)
+	if (!cfgd_enable_plock || ls->disable_plock)
 		return;
 
 	/* if we're the only node in the lockspace, then we are the ckpt_node
@@ -1294,8 +1325,8 @@ static void prepare_plocks(struct lockspace *ls)
 	   previous ckpt_node upon receiving the stored message from us. */
 
 	if (nodes_added(ls))
-		store_plocks(ls);
-	send_plocks_stored(ls);
+		store_plocks(ls, &sig);
+	send_plocks_stored(ls, sig);
 }
 
 static void apply_changes(struct lockspace *ls)
@@ -1529,6 +1560,7 @@ static void dlm_header_in(struct dlm_header *hd)
 	hd->global_id   = le32_to_cpu(hd->global_id);
 	hd->flags       = le32_to_cpu(hd->flags);
 	hd->msgdata     = le32_to_cpu(hd->msgdata);
+	hd->msgdata2    = le32_to_cpu(hd->msgdata2);
 }
 
 static void deliver_cb(cpg_handle_t handle,
@@ -1574,6 +1606,8 @@ static void deliver_cb(cpg_handle_t handle,
 		break;
 
 	case DLM_MSG_PLOCK:
+		if (ls->disable_plock)
+			break;
 		if (cfgd_enable_plock)
 			receive_plock(ls, hd, len);
 		else
@@ -1582,6 +1616,8 @@ static void deliver_cb(cpg_handle_t handle,
 		break;
 
 	case DLM_MSG_PLOCK_OWN:
+		if (ls->disable_plock)
+			break;
 		if (cfgd_enable_plock && cfgd_plock_ownership)
 			receive_own(ls, hd, len);
 		else
@@ -1591,6 +1627,8 @@ static void deliver_cb(cpg_handle_t handle,
 		break;
 
 	case DLM_MSG_PLOCK_DROP:
+		if (ls->disable_plock)
+			break;
 		if (cfgd_enable_plock && cfgd_plock_ownership)
 			receive_drop(ls, hd, len);
 		else
@@ -1601,6 +1639,8 @@ static void deliver_cb(cpg_handle_t handle,
 
 	case DLM_MSG_PLOCK_SYNC_LOCK:
 	case DLM_MSG_PLOCK_SYNC_WAITER:
+		if (ls->disable_plock)
+			break;
 		if (cfgd_enable_plock && cfgd_plock_ownership)
 			receive_sync(ls, hd, len);
 		else
@@ -1610,6 +1650,8 @@ static void deliver_cb(cpg_handle_t handle,
 		break;
 
 	case DLM_MSG_PLOCKS_STORED:
+		if (ls->disable_plock)
+			break;
 		if (cfgd_enable_plock)
 			receive_plocks_stored(ls, hd, len);
 		else
@@ -2344,6 +2386,7 @@ int set_fs_notified(struct lockspace *ls, int nodeid)
 		return -EAGAIN;
 	}
 
+	log_group(ls, "set_fs_notified nodeid %d", nodeid);
 	node->fs_notified = 1;
 	return 0;
 }
